@@ -2,16 +2,21 @@ package Amazon::Credentials;
 
 use strict;
 use warnings;
+use 5.012;
 
-use parent qw/Class::Accessor Exporter/;
+use parent qw/Class::Accessor::Fast/;
 
 __PACKAGE__->follow_best_practice;
-__PACKAGE__->mk_accessors(qw/aws_secret_access_key aws_access_key_id token region
-			     user_agent profile debug expiration role container order 
-			     serialized logger timeout
-			    /);
+
+__PACKAGE__->mk_accessors(
+  qw/aws_secret_access_key aws_access_key_id token region
+    user_agent profile debug expiration role container order
+    serialized logger timeout
+    /
+);
 
 use Carp;
+use Config::Tiny;
 use Data::Dumper;
 use Date::Format;
 use English qw {-no_match_vars};
@@ -28,12 +33,9 @@ use Scalar::Util qw/reftype/;
 use constant  AWS_IAM_SECURITY_CREDENTIALS_URL       => 'http://169.254.169.254/latest/meta-data/iam/security-credentials/';
 use constant  AWS_AVAILABILITY_ZONE_URL              => 'http://169.254.169.254/latest/meta-data/placement/availability-zone';
 use constant  AWS_CONTAINER_CREDENTIALS_URL          => 'http://169.254.170.2';
+use constant  DEFAULT_SEARCH_ORDER                   => 'env,container,role,file';
 
-use vars qw/$VERSION @EXPORT/;
-
-$VERSION = '1.0.15'; $VERSION=~s/\-.*$//;
-
-@EXPORT = qw/$VERSION/;
+our $VERSION = '1.0.16'; $VERSION=~s/\-.*$//;
 
 # we only log at debug level, create a default logger
 {
@@ -66,6 +68,9 @@ sub new {
   $self->set_profile($ENV{AWS_PROFILE})
     if  !$self->get_profile;
 
+  $self->set_order(DEFAULT_SEARCH_ORDER)
+    if !$self->get_order;
+  
   if ( !$self->get_aws_secret_access_key || !$self->get_aws_access_key_id ) {
     $self->set_credentials;
   }
@@ -104,8 +109,9 @@ sub get_default_region {
 }
 
 sub set_credentials {
-  my $self = shift;
-  my $creds = shift || $self->get_ec2_credentials();
+  my ($self, $creds) = @_;
+  
+  $creds = $creds || $self->find_credentials;
   
   if ( $creds->{aws_secret_access_key} && $creds->{aws_access_key_id} ) {
     
@@ -124,20 +130,31 @@ sub get_ec2_credentials {
 }
 
 sub find_credentials {
-  my $self    = shift;
-  my %options = ref( $_[0] ) ? %{ $_[0] } : @_;
+  my ($self, @args) = @_;
+  
+  my $options = ref($args[0]) ? $args[0] : { @args };
 
-  $options{order}   = $self->get_order  || [qw/env role container file/];
-  $options{profile} = $options{profile} || $self->get_profile;
+  $self->set_profile($options->{profile})
+    if $options->{profile};
 
-  if ( defined $options{profile} && $options{profile} ) {
-    $options{order} = ['file'];
+  $self->set_order($options->{order})
+    if $options->{order};
+
+  my @search_order;
+  
+  if ( $self->get_profile ) {
+    @search_order = ('file');
   }
-
+  elsif ( ref($self->get_order) && reftype($self->get_order) eq 'ARRAY' ) {
+    @search_order = @{$self->get_order};
+  }
+  elsif (!ref($self->get_order)) {
+    @search_order = split /\s*,\s*/, $self->get_order;
+  }
+         
   my $creds = {};
-  my @order = @{ $options{order} };
 
-  foreach my $location (@order) {
+  foreach my $location (@search_order) {
     
     if ( $location eq 'env' ) {
       
@@ -146,102 +163,17 @@ sub find_credentials {
           = (
           'ENV',
           @ENV{qw/AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY AWS_SESSION_TOKEN/}
-            );
-        $creds->{source} = 'env';
+          );
       } ## end if ( $ENV{AWS_ACCESS_KEY_ID...})
     } ## end if ( $location eq 'env')
     elsif ( $location eq 'container' ) {
       $creds = $self->get_creds_from_container();
     }
     elsif ( $location eq 'role' ) {
-      $creds = $self->get_creds_from_role();
+      $creds = $self->get_creds_from_role;
     }
     elsif ( $location eq 'file' ) {
-      # look for ~/.aws/config and/or .aws/credentials
-
-      # in production environments, we should never be reading creds
-      # from files, so let's require these (not use)
-
-      foreach my $config ( ".aws/config", ".aws/credentials" ) {
-        # reset this since we have hav
-        my $profile_name = $options{profile};
-
-        local $CWD = home;
-
-        next unless -e $config;
-
-        open my $fh, '<', $config
-          or croak 'could not open credentials file!';
-
-        my $last_profile    = '';
-        my $profile_to_find = $profile_name;
-
-        # look for credentials...by interating through credentials file
-        while (my $current_line = <$fh>) {
-          $self->get_logger->debug('looking for profile ' . ($profile_name // '') . "\n");
-            
-          chomp $current_line;
-
-          # once we find a profile section that matches, undef it
-          # ./aws/credentials uses [profile-name]
-          # ./aws/config uses [profile profile-name]
-
-          if ($current_line =~/^\s*region\s*=\s*(.*?)\s*$/) {
-            my $region = $1;
-            # go ahead and use this region setting IF:
-            # 1. this is the default profile (we may reset region later though)
-            # 2. the profile we want to use is this profile
-            # 3. we are not in a profile at all (this is yet another default)
-            if ( $last_profile =~ /default/
-              || ( $profile_to_find && $last_profile =~ /$profile_to_find/ )
-              || !$last_profile ) {
-              $self->set_region($region);
-            }
-          }
-
-          if ($profile_name) {
-            if ($current_line =~ /^\s*\[\s*profile\s+$profile_name\s*\]/) {
-              undef $profile_name;
-            }
-            elsif ($current_line =~/^\s*\[\s*$profile_name\s*\]/) {
-              undef $profile_name;
-              $last_profile = $current_line;
-            }
-          } ## end if ($profile_name)
-          elsif ( defined $profile_name && $current_line =~/^\s*\[\s*profile\s+/ ) {
-            last;
-          }
-          elsif ($current_line =~/credential_process/ ) {
-            my $process;
-            if ( $current_line =~/\=(.*)$/ ) {
-              $process = $1;
-            }
-            
-            @{$creds}{qw{aws_access_key_id aws_secret_access_key token}} =
-              $self->get_credentials_from_process($process);
-
-            $creds->{source} = 'process';
-            last;
-          }
-          elsif ($current_line =~/^\s*aws_secret_access_key\s*=\s*(.*)$/) {
-            last if defined $creds->{aws_secret_access_key};    # next profile
-            $creds->{aws_secret_access_key} = $1;
-          }
-          elsif ($current_line =~/^\s*aws_access_key_id\s*=\s*(.*)$/) {
-            last if defined $creds->{aws_access_key_id};        # next profile
-            $creds->{aws_access_key_id} = $1;
-          }
-          elsif ($current_line =~/^\s*aws_session_token\s*=\s*(.*)$/) {
-            last if defined $creds->{token};
-            $creds->{token} = $1;
-          }
-        } ## end while (<$fh>)
-
-        close $fh;
-
-        $creds->{source} = $config
-          if $creds->{aws_secret_access_key} && $creds->{aws_access_key_id};
-      } ## end foreach my $config ( ".aws/config"...)
+      $creds = $self->get_creds_from_ini_file;
     } ## end elsif ( $location eq 'file')
 
     last if $creds->{source};
@@ -255,6 +187,167 @@ sub find_credentials {
 } ## end sub find_credentials
 
 
+sub get_creds_from_file {
+  my ($self, $profile) = @_;
+
+  $profile = $profile || $self->get_profile || 'default';
+  
+  my $creds = {};
+  
+  foreach my $config ( ".aws/config", ".aws/credentials" ) {
+    my $profile_name = $profile;
+
+    local $CWD = home;
+
+    next if !-e $config;
+
+    open my $fh, '<', $config
+      or croak 'could not open credentials file!';
+
+    my $last_profile = '';
+    my $profile_to_find = $profile_name;
+
+    # look for credentials...by interating through credentials file
+    # (could use Config::INI but did not want to add another
+    # dependency), maybe Config::Tiny?
+    while ( my $current_line = <$fh> ) {
+
+      chomp $current_line;
+
+      $self->get_logger->debug(
+        'looking for profile ' . ( $profile_name // '' ) . "\n" );
+
+      # once we find a profile section that matches, undef it
+      # ./aws/credentials uses [profile-name]
+      # ./aws/config uses [profile profile-name]
+
+      if ( $current_line =~ /^\s*region\s*=\s*(.*?)\s*$/ ) {
+        my $region = $1;
+        # go ahead and use this region setting IF:
+        # 1. this is the default profile (we may reset region later though)
+        # 2. the profile we want to use is this profile
+        # 3. we are not in a profile at all (this is yet another default)
+        if ( $last_profile =~ /default/
+          || ( $profile_to_find && $last_profile =~ /$profile_to_find/ )
+          || !$last_profile ) {
+          $self->set_region($region);
+        }
+      } ## end if ( $current_line =~ ...)
+
+      if ($profile_name) {
+        if ( $current_line =~ /^\s*\[\s*profile\s+$profile_name\s*\]/ ) {
+          undef $profile_name;
+        }
+        elsif ( $current_line =~ /^\s*\[\s*$profile_name\s*\]/ ) {
+          undef $profile_name;
+          $last_profile = $current_line;
+        }
+      } ## end if ($profile_name)
+      elsif ( defined $profile_name
+        && $current_line =~ /^\s*\[\s*profile\s+/ ) {
+        last;
+      }
+      elsif ( $current_line =~ /credential_process/ ) {
+        my $process;
+        if ( $current_line =~ /\=(.*)$/ ) {
+          $process = $1;
+        }
+
+        @{$creds}{qw{source aws_access_key_id aws_secret_access_key token}}
+          = ('process', $self->get_creds_from_process($process));
+
+        last;
+      } ## end elsif ( $current_line =~ ...)
+      elsif ( $current_line =~ /^\s*aws_secret_access_key\s*=\s*(.*)$/ ) {
+        last if defined $creds->{aws_secret_access_key};    # next profile
+        $creds->{aws_secret_access_key} = $1;
+      }
+      elsif ( $current_line =~ /^\s*aws_access_key_id\s*=\s*(.*)$/ ) {
+        last if defined $creds->{aws_access_key_id};        # next profile
+        $creds->{aws_access_key_id} = $1;
+      }
+      elsif ( $current_line =~ /^\s*aws_session_token\s*=\s*(.*)$/ ) {
+        last if defined $creds->{token};
+        $creds->{token} = $1;
+      }
+      elsif ( $current_line =~ /^\s*profile\s*=\s*(.*)$/ ) {
+        $profile_name = $1;
+        seek($fh, 0, 1);
+      }
+    } ## end while ( my $current_line ...)
+
+    close $fh;
+
+    $creds->{source} = $config
+      if $creds->{aws_secret_access_key} && $creds->{aws_access_key_id};
+  } ## end foreach my $config ( ".aws/config"...)
+
+  return $creds;
+} ## end sub get_credentials_from_file
+
+sub get_creds_from_ini_file {
+  my ($self, $profile) = @_;
+  
+  $profile = $profile || $self->get_profile || 'default';
+
+  my $creds = {};
+  
+  foreach my $config ( ".aws/config", ".aws/credentials" ) {
+    last if $creds->{source};
+    
+    my $profile_name = $profile;
+
+    local $CWD = home;
+
+    next if !-e $config;
+
+    my $ini = Config::Tiny->read($config);
+
+    if ($profile eq 'default') {
+      next if ! $ini->{default}->{profile};
+      
+      $profile_name = $ini->{default}->{profile};
+    }
+    
+    $self->set_region($ini->{region})
+      if $ini->{region};
+    
+    my $section;
+    
+    if ( $ini->{$profile_name} ) {
+      $section = $ini->{$profile_name};
+    }
+    elsif ($ini->{"profile $profile_name"}) {
+      $section = $ini->{"profile $profile_name"};
+    }
+    
+    if ($ini->{credential_process}) {
+      @{$creds}{qw{source aws_access_key_id aws_secret_access_key token region}}
+        = ('process', $self->get_creds_from_process($ini->{credential_process}));
+    }
+
+    if ( $section ) {
+      if ($section->{credential_process}) {
+        @{$creds}{qw{source aws_access_key_id aws_secret_access_key token region}}
+          = ('process', $self->get_creds_from_process($section->{credential_process}));
+      }
+      else {
+        foreach my $k (qw{aws_access_key_id aws_secret_access_key token region}) {
+          $creds->{$k} = $section->{$k};
+        }
+      }
+      
+      $creds->{source} = $config
+        if $creds->{aws_access_key_id};
+    }
+  }
+
+  $self->set_region($creds->{region})
+    if $creds->{region};
+
+  return $creds;
+} ## end sub get_credentials_from_file
+
 sub is_token_expired {
   my $self = shift;
   my $window_interval = shift || 5;
@@ -352,7 +445,7 @@ sub get_creds_from_role {
   $creds;
 }
 
-sub get_credentials_from_process {
+sub get_creds_from_process {
   my ($self, $process) = @_;
 
   open(my $fh, '-|', $process)
@@ -366,12 +459,12 @@ sub get_credentials_from_process {
   $credentials = eval {
     JSON::PP->new->decode($credentials);
   };
-  
+
   if ( $EVAL_ERROR || !$credentials ) {
     die "could not get credentials from process\n$EVAL_ERROR\n";
   }
   
-  return @{$credentials}{qw/AccessKeyId SecretAccessKey SessionToken/};
+  return @{$credentials}{qw/AccessKeyId SecretAccessKey SessionToken Region Expiration/};
 }
 
 sub refresh_credentials {
@@ -482,11 +575,12 @@ meta-data>. See L</new>.
 
  my $aws_creds = Amazon::Credential->new( { profile => 'sandbox', debug => 1 });
 
-C<options> is a hash of keys that represent options.  Any of the
-options can also be retrieved using their corresponding 'get_{option}
-method.
+C<options> is a hash of keys that represent various options you can
+pass to the constructor to control how it will look for credentials.
+Any of the options can also be retrieved using their corresponding
+'get_{option} method.
 
-Options are listed below.
+=head3 options
 
 =over 5
 
